@@ -28,9 +28,8 @@ from ._device import Device
 from ._imagemounter import cache_developer_image
 from ._ipautil import IPAReader
 from ._perf import DataType
-from ._proto import LOG, MODELS, PROGRAM_NAME
+from ._proto import LOG, MODELS, PROGRAM_NAME, ConnectionType
 from ._relay import relay
-from ._types import ConnectionType
 from ._usbmux import Usbmux
 from ._utils import get_app_dir, get_binary_by_name, is_atty
 from ._version import __version__
@@ -261,20 +260,45 @@ def cmd_xctest(args: argparse.Namespace):
         env[key] = val
     if env:
         logger.info("Launch env: %s", env)
+
+    test_runner_args = []
+    if args.test_runner_args:
+        test_runner_args = args.test_runner_args.split(',')
+        logger.info("Launch test runner args: %s", test_runner_args)
+
+    target_app_env = dict(
+        token.split(":", 1) for token in args.target_app_env or [])
+    if target_app_env:
+        logger.info("Target app env: %s", target_app_env)
+
+    target_app_args = []
+    if args.target_app_args:
+        target_app_args = args.target_app_args.split(',')
+        logger.info("Target app args: %s", target_app_args)
+
+    tests_to_run = set()
+    if args.tests_to_run:
+        tests_to_run = set(args.tests_to_run.strip().split(','))
+        logger.info("Target app args: %s", target_app_args)
+
     d.xctest(args.bundle_id,
              target_bundle_id=args.target_bundle_id,
              logger=setup_logger(level=logging.INFO),
-             env=env)
+             test_runner_env=env,
+             test_runner_args=test_runner_args,
+             target_app_env=target_app_env,
+             target_app_args=target_app_args,
+             tests_to_run=tests_to_run)
 
 
 def cmd_screenshot(args: argparse.Namespace):
     d = _udid2device(args.udid)
     filename = args.filename or "screenshot.jpg"
-    print("Screenshot saved to", filename)
     d.screenshot().convert("RGB").save(filename)
+    print("Screenshot saved to", filename)
 
 
-def cmd_app_info(args: argparse.Namespace):
+def cmd_appinfo(args: argparse.Namespace):
     d = _udid2device(args.udid)
     info = d.installation.lookup(args.bundle_id)
     if info is None:
@@ -343,7 +367,7 @@ def cmd_launch(args: argparse.Namespace):
             pid = ts.app_launch(args.bundle_id,
                                         app_env=env,
                                         args=args.arguments,
-                                        kill_running=args.kill)
+                                        kill_running=not args.skip_running)
             print("PID:", pid)
     except ServiceError as e:
         sys.exit(e)
@@ -405,7 +429,7 @@ def cmd_battery(args: argparse.Namespace):
             else:
                 value = info
                 for key in keypath:
-                    value = value[key]
+                    value = value.get(key, {})
             if callable(unit):
                 value = unit(value)
                 unit = ""
@@ -527,15 +551,19 @@ def cmd_fsync(args: argparse.Namespace):
 
     arg0 = args.arguments[0]
     if args.command == 'ls':
-        for filename in sync.listdir(arg0):
-            finfo = sync.stat(pathlib.Path(arg0) / filename)
+        tabdata = []
+        for finfo in sync.listdir_info(arg0):
+            filename = finfo.st_name
             if finfo.is_dir():
-                print(filename+"/")
-            else:
-                print("-", filename, finfo.st_size)
-    elif args.command == 'rm':
+                filename = filename + "/"
+            tabdata.append(['d' if finfo.is_dir() else '-', finfo.st_size, finfo.st_mtime, filename])
+        print(tabulate(tabdata,  tablefmt="plain"))
+    elif args.command == 'rm': # rm also support rmdir
         for arg in args.arguments:
             pprint(sync.remove(arg))
+    elif args.command == "touch":
+        for arg in args.arguments:
+            pprint(sync.touch(arg))
     elif args.command == 'stat':
         finfo = sync.stat(arg0)
         print("IFMT:", finfo.st_ifmt)
@@ -608,7 +636,6 @@ def cmd_ps(args: argparse.Namespace):
 
 
 def cmd_perf(args: argparse.Namespace):
-    assert args.bundle_id
     #print("BundleID:", args.bundle_id)
     from ._perf import Performance
     d = _udid2device(args.udid)
@@ -617,21 +644,28 @@ def cmd_perf(args: argparse.Namespace):
         perfs = []
         for _typename in args.perfs.split(","):
             perfs.append(DataType(_typename))
-    # print(perfs)
+    
+    if (DataType.MEMORY in perfs or DataType.CPU in perfs) and not args.bundle_id:
+        print('\033[1;31m error: the following arguments are required: -B/--bundle_id \033[0m')
+        exit(-1)
+
     perf = Performance(d, perfs=perfs)
 
     def _cb(_type: DataType, data):
-        print(_type.value, data, flush=True)
+        if args.json and _type != DataType.SCREENSHOT:
+            data = json.dumps(data)
+        print(_type.value, data, flush=True) if len(perfs) != 1 else print(data,flush=True)
 
     try:
         perf.start(args.bundle_id, callback=_cb)
         #print("Ctrl-C to finish")
         while True:
             time.sleep(.1)
+    except KeyboardInterrupt:
+        pass
     finally:
         perf.stop()
 
-
 def cmd_set_assistive_touch(args: argparse.Namespace):
     d = _udid2device(args.udid)
     d.set_assistive_touch(args.enabled)
@@ -699,7 +733,7 @@ _commands = [
     dict(action=cmd_system_info,
          command="sysinfo",
          help="show device system info"),
-    dict(action=cmd_app_info,
+    dict(action=cmd_appinfo,
          command="appinfo",
          flags=[
              dict(args=['--json'],
@@ -758,9 +792,7 @@ _commands = [
     dict(action=cmd_launch,
          command="launch",
          flags=[
-             dict(args=['--kill'],
-                  action='store_true',
-                  help='kill app if running'),
+             dict(args=['--skip-running'], action='store_true', help='not kill app if running'),
              dict(args=["bundle_id"], help="app bundleId"),
              dict(args=['arguments'], nargs='*', help='app arguments'),
              dict(args=['-e', '--env'],
@@ -806,6 +838,7 @@ _commands = [
     dict(
         action=cmd_xctest,
         command="xctest",
+        aliases=['xcuitest'],
         flags=[
             dict(args=['--debug'], action='store_true', help='show debug log'),
             dict(args=['-B', '--bundle_id', '--bundle-id'],
@@ -818,9 +851,18 @@ _commands = [
             #       help='install webdriveragent app'),
             dict(args=['-e', '--env'],
                  action='append',
-                 help="set env with format key:value, support multi -e"),
+                 help="set test runner env with format key:value, support multi -e"),
+            dict(args=['--test-runner-args'],
+                 help="set command line args to test runner with a comma-separated list of strings"),
+            dict(args=['--target-app-env'],
+                 action='append',
+                 help="set env to target app with format key:value, support multi --target-app-env"),
+            dict(args=['--target-app-args'],
+                 help="set command line args to target app with a comma-separated list of strings"),
+            dict(args=['--tests-to-run'],
+                 help="specify a set of test classes or test methods to run, format: a comma-separated list of Test-Class-Name[/Test-Method-Name]"),
         ],
-        help="run XCTest"),
+        help="run XCTest (XCUITest)"),
     dict(
         action=cmd_wdaproxy,
         command='wdaproxy',
@@ -855,7 +897,7 @@ _commands = [
              dict(args=['command'],
                   choices=[
                       'ls', 'rm', 'cat', 'pull', 'push', 'stat', 'tree',
-                      'rmtree', 'mkdir'
+                      'rmtree', 'mkdir', 'touch'
                   ]),
              dict(args=['arguments'], nargs='+', help='command arguments'),
          ],
@@ -895,12 +937,15 @@ _commands = [
          command="perf",
          flags=[
              dict(args=['-B', '--bundle_id'],
-                  help='app bundle id',
-                  required=True),
+                  help='app bundle id (cpu/memory required)',
+                  required=False),
              dict(args=['-o'],
                   dest='perfs',
                   help='cpu,memory,fps,network,screenshot. separate by ","',
                   required=False),
+            dict(args=['--json'],
+                  action='store_true',
+                  help='format output as json'),
          ],
          help="performance of app"),
     dict(action=cmd_set_assistive_touch,
@@ -930,8 +975,10 @@ def main():
     actions = {}
     for c in _commands:
         cmd_name = c['command']
-        actions[cmd_name] = c['action']
-        sp = subparser.add_parser(cmd_name, help=c.get('help'),
+        cmd_aliases = c.get('aliases', [])
+        for alias in [cmd_name] + cmd_aliases:
+            actions[alias] = c['action']
+        sp = subparser.add_parser(cmd_name, aliases=cmd_aliases, help=c.get('help'),
                                   formatter_class=argparse.ArgumentDefaultsHelpFormatter)
         for f in c.get('flags', []):
             args = f.get('args')
