@@ -11,7 +11,7 @@ from typing import List
 import retry
 import requests
 
-from ._safe_socket import PlistSocketProperty
+from ._safe_socket import PlistSocketProxy
 from ._utils import get_app_dir, logger
 from .exceptions import MuxError, MuxServiceError
 
@@ -42,12 +42,9 @@ def get_developer_image_url_list(version: str) -> typing.List[str]:
     """ return url list which may contains mirror url """
     # https://github.com/JinjunHan/iOSDeviceSupport
     github_repo = "JinjunHan/iOSDeviceSupport"
-    zip_name = f"{version}.zip"
 
-    # the code.aliyun slowlly
-    # gitee requires login
-    # aliyun_url = f"https://code.aliyun.com/hanjinjun/iOSDeviceSupoort/raw/master/DeviceSupport/{zip_name}"
-    origin_url = f"https://github.com/{github_repo}/raw/master/DeviceSupport/{zip_name}"
+    zip_name = f"{version}.zip"
+    origin_url = f"https://github.com/{github_repo}/raw/master/iOSDeviceSupport/{zip_name}"
     mirror_url = origin_url.replace("https://github.com", "https://tool.appetizer.io")
     return (mirror_url, origin_url)
 
@@ -58,7 +55,20 @@ def cache_developer_image(version: str) -> str:
     """
     _alias = {
         "12.5": "12.4",
+        "16.6": "16.5",
     }
+    # https://github.com/alibaba/taobao-iphone-device/issues/258#issuecomment-1328728799
+    # download image from https://github.com/iGhibli/iOS-DeviceSupport
+    # 换了16.1好像也没什么卵用
+    version_url_map = {
+        "15.7": "https://github.com/iGhibli/iOS-DeviceSupport/raw/master/DeviceSupport/15.7(FromXcode_14.1_Release_Candidate_xip).zip",
+    }
+
+    # Default download image from https://github.com/JinjunHan/iOSDeviceSupport
+    image_urls = get_developer_image_url_list(version)
+    if version in version_url_map:
+        image_urls = [version_url_map[version]]
+
     if version in _alias:
         version = _alias[version]
         logger.info("Use alternative developer image %s", version)
@@ -67,10 +77,8 @@ def cache_developer_image(version: str) -> str:
     local_device_support = get_app_dir("device-support")
     image_zip_path = os.path.join(local_device_support, version+".zip")
     if not zipfile.is_zipfile(image_zip_path):
-        urls = get_developer_image_url_list(version)
-
         err = None
-        for url in urls:
+        for url in image_urls:
             try:
                 _urlretrieve(url, image_zip_path)
                 if zipfile.is_zipfile(image_zip_path):
@@ -88,21 +96,20 @@ def cache_developer_image(version: str) -> str:
     return image_zip_path
 
 
-class ImageMounter(PlistSocketProperty):
+class ImageMounter(PlistSocketProxy):
     SERVICE_NAME = "com.apple.mobile.mobile_image_mounter"
 
     def prepare(self):
         """
         Note: LookupImage might stuck and no response
         """
-        self.psock.get_socket().settimeout(10.0)
         return super().prepare()
     
     def lookup(self, image_type="Developer") -> List[bytes]:
         """
         Check image signature
         """
-        ret = self.psock.send_recv_packet({
+        ret = self.send_recv_packet({
             "Command": "LookupImage",
             "ImageType": image_type,
         })
@@ -145,7 +152,7 @@ class ImageMounter(PlistSocketProperty):
                 signature_content: bytes,
                 image_type: str = "Developer"):
 
-        ret = self.psock.send_recv_packet({
+        ret = self.send_recv_packet({
             "Command": "ReceiveBytes",
             "ImageSignature": signature_content,
             "ImageSize": image_size,
